//
//  SXTrackGroup.h
//  SXVideoEnging
//
//  Created by Yin Xie on 2020/3/31.
//  Copyright © 2020 Zhiqiang Li. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SXTrack.h"
NS_ASSUME_NONNULL_BEGIN

/**
 * 轨道组，可以包含多个轨道
 * @warning 注意所有的对象对比是否相等时请使用id对比，引用可能会不同
*/
@interface SXTrackGroup : NSObject

/**
 * 该轨道是否是主轨道
 * @return YES:主轨道组 NO：其他轨道组
 */
- (BOOL)isMainTrackGroup;

/**
 * 在组的某个时间添加一个轨道，如无法容纳此轨道返回false
 * @param track 轨道
 * @param time 相对于组的时间线
 * @return 添加成功返回true，失败返回false
 */
- (BOOL)addTrack:(SXTrack *)track inPoint:(double)time;


/**
 * 从组中移除一个轨道
 * @param trackId 轨道唯一id
 * @return 如组内有此轨道返回true，无此轨道返回false
 */
- (BOOL)removeTrack:(NSString *)trackId;


/**
 * 清空组内所有轨道
 */
- (void)clearTracks;


/**
 * start time    in point        duration      out point    end time
 *    |             |               |             |            |
 *    |             |-----------------------------|            |
 *    |-------------|-----------------------------|------------|
 * ----------------------------------------------------------------------
 * 设置轨道的开始显示时间，调用该方法会同步修改duration
 * @param trackId 轨道唯一id
 * @param time 相对于组的时间线
 * @return 查找轨道失败、in point超出start time或 out point均会返回false，否则返回true
 */
- (BOOL)setTrack:(NSString *)trackId inPoint:(double)time;


/**
 * start time    in point        duration      out point    end time
 *    |             |               |             |            |
 *    |             |-----------------------------|            |
 *    |-------------|-----------------------------|------------|
 * ----------------------------------------------------------------------
 * 设置轨道的显示时长，允许时长超出end time，超出end time则自动循环
 * @param trackId 轨道唯一id
 * @param time 相对于组的时间线
 * @return 查找轨道失败或时长小于0返回false，否则返回true
 */
- (BOOL)setTrack:(NSString *)trackId duration:(double)time;


/**
 * start time    in point        duration      out point    end time
 *    |             |               |             |            |
 *    |             |-----------------------------|            |
 *    |-------------|-----------------------------|------------|
 * ----------------------------------------------------------------------
 * 整体移动轨道
 * @param trackId 轨道唯一id
 * @param time 偏移时长
 * @return 查找轨道失败或偏移后无法容纳此轨道返回false，否则返回true
 */
- (BOOL)setTrack:(NSString *)trackId offset:(double)time;

/**
 * start time    in point        duration      out point    end time
 *    |             |               |             |            |
 *    |             |-----------------------------|            |
 *    |-------------|-----------------------------|------------|
 * ----------------------------------------------------------------------
 * 设置轨道时间信息
 * @param trackId 轨道唯一id
 * @param start 轨道原素材起始点相对于组的时间线
 * @param inpoint 轨道显示起始点相对于组的时间线
 * @param duration 轨道显示时长
 * @return 查找轨道失败或时间改变后无法容纳此轨道返回false，否则返回true
 */
- (BOOL)setTrack:(NSString *)trackId start:(double)start inpoint:(double)inpoint duration:(double)duration;
/**
 * 通过id查找组中此轨道所在的下一个轨道
 * @param trackId 轨道唯一id
 * @return 查找成功返回轨道实例对象，否则返回nullptr
 */
- (SXTrack *)findNextTrack:(NSString *)trackId;


/**
 * 通过id查找组中此轨道所在的上一轨道
 * @param trackId 轨道唯一id
 * @return 查找成功返回轨道实例对象，否则返回nullptr
 */
- (SXTrack *)findPreviousTrack:(NSString *)trackId;


/**
 * 通过id查找组内的轨道
 * @param trackId 轨道唯一id
 * @return 查找成功返回轨道实例对象，否则返回nullptr
 */
- (SXTrack *)track:(NSString *)trackId;


/**
 * in point   track     split time     new track    out point
 *   |-----------------------|--------------------------|
 * ----------------------------------------------------------------------
 * 拆分轨道，拆分后的轨道附加效果会被移除或者缩短时间，请重新获取相关数据
 * @note 拆分后的新轨道会自动加入此组中
 * @param trackId 轨道唯一id
 * @param time 相对于轨道自身in point拆分时间点
 * @return 查找轨道失败或拆分时间不在轨道显示范围内返回nullptr，否则返回轨道实例对象
 */
- (SXTrack *)splitTrack:(NSString *)trackId time:(double)time;

/**
 * 设置媒体轨道变速，变速后的轨道依然不允许与其他轨道重叠，并最少保留1帧
 * @warning 仅媒体轨道（media track）可用
 * @param trackId 轨道id
 * @param speed 设置播放倍速
 * @param realSpeed 实际播放倍速
 * @return 是否设置成功，非媒体轨道或轨道查找失败会导致设置失败
 */
- (BOOL)setMediaTrackSpeed:(NSString *)trackId speed:(double)speed realSpeed:(double *)realSpeed;

/**
 * 获取媒体轨道播放倍速
 * @warning 仅媒体轨道（media track）可用
 * @param trackId 轨道id
 * @return 媒体轨道播放倍速，失败返回0
 */
- (double)mediaTrackSpeed:(NSString *)trackId;

/**
 * 获取变速曲线
 * @return 变速曲线对象，无曲线变速时返回nil
 * @param trackId 轨道id
 */
- (UIBezierPath *)mediaTrackSpeedCurve:(NSString *)trackId;

/**
 * 设置媒体轨道曲线变速，变速后的轨道依然不允许与其他轨道重叠，并最少保留1帧
 * @warning 变速后影响轨道时长，仅媒体轨道（media track）视频或GIF可用，图片不支持变速速
 * @param track_id 轨道id
 * @param speedCurve 变速曲线
 * @return 是否设置成功，非媒体轨道、轨道查找失败获取变速后违法组内规则会导致设置失败
 */
- (BOOL)setMediaTrackSpeed:(NSString *)trackId path:(UIBezierPath *)path;

/**
 * 获取所有轨道
 * @return 轨道链表
 */
- (NSArray *)tracks;


/**
 * 获取轨道时长
 * @return 时长
 */
- (double)duration;


/**
 * 轨道唯一id
 * @return id
 */
- (NSString *)getGroupId;

@end

#pragma mark - Main Group
@interface SXMainTrackGroup : SXTrackGroup <SXVideoEffectDelegate, SXColorEffectDelegate, SXFilterDelegate>

/**
 * 在轨道后附加一个转场效果，转场仅存在于轨道末尾，时长受自身轨道时长与下一个相衔接的轨道显示时长影响
 * 如果此轨道已存在转场则会替换调已存在的转场
 * @warning 转场时长不得大于前后轨道显示时长最小值的一半
 * @param transitionPath 转场资源，资源类型需为转场且资源有效
 * @param trackId 轨道id
 * @param duration 转场时长
 * @param error 错误码
 * @return 转场实际时长，添加失败返回0
 */
- (double)addTransition:(NSString *)transitionPath
                trackId:(NSString *)trackId
               duration:(double)duration
                  error:(int *)error;

/**
 * 设置转场时长，转场时长受自身轨道时长与下一个相衔接的轨道显示时长影响
 * @warning 转场时长不得大于前后轨道显示时长最小值的一半
 * @param trackId 轨道id
 * @param duration 转场时长
 * @return 实际时长
 */
- (double)setTransitionTrackId:(NSString *)trackId duration:(double)duration;

/**
 * 获取轨道所附加的转场时间，转场时长受前后轨道时长影响，因此在前后轨道发生变化或
 * 轨道时长发生变化时，转场时长也可能会发生变化
 * @param trackId 轨道id
 * @return 转场时长
 */
- (double)getTransitionDuration:(NSString *)trackId;

/**
 * 获取轨道所附加的转场最大时长，转场时长受前后轨道时长影响，因此在前后轨道发生变化或
 * 轨道时长发生变化时，转场时长也可能会发生变化。转场最大时长不超过转场前后轨道最小时长的一半。
 * 当转场后无轨道时，最大时长为所附加轨道时长的一半
 * @param trackId 轨道id
 * @return 转场最大时长
 */
- (double)getTransitionMaxDuration:(NSString *)trackId;

/**
 * 移除转场
 * @param trackId 轨道id
 */
- (void)removeTransition:(NSString *)trackId;

/**
 * 查询轨道是否有转场
 * @param trackId 轨道id
 * @return 查询轨道成功且有转场返回true，否则返回false
 */
- (BOOL)hasTransition:(NSString *)trackId;

/**
 * 获取轨道所附加的转场资源
 * @param trackId 轨道id
 * @return 转场资源路径，如轨道查找失败或无转场资源返回nil
 */
- (NSString *)getTranistionResourcePath:(NSString *)trackId;
@end

NS_ASSUME_NONNULL_END
